@@ -10,7 +10,11 @@
 ac_help='--enable-amalloc	Enable memory allocation debugging
 --with-tabstops=N	Set tabstops to N characters (default is 4)
 --shared		Build shared libraries (default is static)
---pkg-config		Install pkg-config(1) glue files'
+--pkg-config		Install pkg-config(1) glue files
+--cxx-binding		Install header files with c++ wrappers
+--github-checkbox[=input] Enable github-style checkboxes in lists
+			(if =input, use <input checkbox>, otherwise
+			use html ballot entities)'
 
 LOCAL_AC_OPTIONS='
 set=`locals $*`;
@@ -32,8 +36,20 @@ locals() {
     --DEBIAN-GLITCH)
 		echo DEBIAN_GLITCH=T
 		;;
+    --H1-TITLE)
+		echo H1TITLE=T
+		;;
     --PKG-CONFIG)
-		echo PKGCONFIG=true
+		echo PKGCONFIG=T
+		;;
+    --CXX-BINDING)
+		echo CXX_BINDING=T
+		;;
+    --GITHUB-CHECKBOX=ENTITY)
+		echo GITHUB_CHECKBOX_STYLE=entity
+		;;
+    --GITHUB-CHECKBOX=INPUT)
+		echo GITHUB_CHECKBOX_STYLE=input
 		;;
     esac
 }
@@ -42,24 +58,24 @@ VERSION=`cat VERSION`
 TARGET=markdown
 . ./configure.inc
 
+# if there's a makefile here, it's likely that it's a discount
+# makefile and there's bits of an old configuration here.  So
+# blow everything away before we start the configuration.
+
+test -f Makefile && make spotless 2>/dev/null >/dev/null
+
 AC_INIT $TARGET
 AC_SUB 'PACKAGE_NAME' lib$TARGET
 AC_SUB 'PACKAGE_VERSION' $VERSION
 
-for banned_with in dl fenced-code id-anchor github-tags urlencoded-anchor; do
-    banned_with_variable_ref=\$WITH_`echo "$banned_with" | $AC_UPPERCASE | tr - _`
-    if [ "`eval echo "$banned_with_variable_ref"`" ]; then
-	LOG "Setting theme default --with-$banned_with."
-    fi
-done
-
-# theme wants the old behavior of --with-(foo)
+# define definition list type defaults (for theme)
 #
 case "`echo "$WITH_DL" | $AC_UPPERCASE`" in
-    EXTRA)         THEME_CF="MKD_DLEXTRA|MKD_NODLDISCOUNT";;
-    BOTH)          THEME_CF="MKD_DLEXTRA";;
+    DISCOUNT)      AC_DEFINE THEME_DL_MODE 1 ;;
+    EXTRA)         AC_DEFINE THEME_DL_MODE 2 ;;
+    BOTH)          AC_DEFINE THEME_DL_MODE 3 ;;
 esac
-test "$WITH_FENCED_CODE" && THEME_CF="${THEME_CF:+$THEME_CF|}MKD_FENCEDCODE"
+test "$WITH_FENCED_CODE" && AC_DEFINE THEME_FENCED_CODE 1
 
 AC_DEFINE THEME_CF "$THEME_CF"
 
@@ -67,7 +83,8 @@ AC_DEFINE THEME_CF "$THEME_CF"
 test "$DEBIAN_GLITCH" && AC_DEFINE 'DEBIAN_GLITCH' 1
 
 AC_PROG_CC
-AC_CHECK_NORETURN
+AC_QUIET AC_PROG git && AC_DEFINE 'HAS_GIT' '1'
+AC_CHECK_ATTRIBUTE destructor
 
 test "$TRY_SHARED" && AC_COMPILER_PIC && AC_CC_SHLIBS
 
@@ -109,6 +126,36 @@ AC_CHECK_BASENAME
 AC_CHECK_ALLOCA
 
 AC_CHECK_HEADERS sys/types.h pwd.h && AC_CHECK_FUNCS getpwuid
+if AC_CHECK_HEADERS sys/stat.h && AC_CHECK_FUNCS stat; then
+
+# need to check some of the S_ISxxx stat macros, because they may not
+# exist (for notspecial.c)
+
+cat > ngc$$.c << EOF
+#include <sys/stat.h>
+
+int main(int argc, char **argv)
+{
+   struct stat info;
+
+    if ( stat(argv[0], &info) != 0 )
+	return 1;
+
+    return MACRO(info.st_mode);
+}
+EOF
+    LOGN "special file macros in sys/stat.h:"
+    _none="none"
+    for x in ISSOCK ISCHR ISFIFO; do
+	if $AC_CC -DMACRO=S_$x -o ngc$$.o  ngc$$.c; then
+	    LOGN " S_${x}"
+	    AC_DEFINE "HAS_${x}" '1'
+	    unset _none
+	fi
+    done
+    LOG "${_none}."
+    __remove ngc$$.o ngc$$.c
+fi
 
 if AC_CHECK_FUNCS srandom; then
     AC_DEFINE 'INITRNG(x)' 'srandom((unsigned int)x)'
@@ -118,13 +165,9 @@ else
     AC_DEFINE 'INITRNG(x)' '(void)1'
 fi
 
-if AC_CHECK_FUNCS 'bzero((char*)0,0)'; then
-    : # Yay
-elif AC_CHECK_FUNCS 'memset((char*)0,0,0)'; then
-    AC_DEFINE 'bzero(p,s)' 'memset(p,s,0)'
-else
-    AC_FAIL "$TARGET requires bzero or memset"
-fi
+AC_CHECK_FUNCS 'memset((char*)0,0,0)' 'string.h' || \
+	    AC_CHECK_FUNCS 'memset((char*)0,0,0)' || \
+		      AC_FAIL "$TARGET requires memset"
 
 if AC_CHECK_FUNCS random; then
     AC_DEFINE 'COINTOSS()' '(random()&1)'
@@ -174,6 +217,20 @@ else
     AC_SUB	'AMALLOC'	''
 fi
 
+if [ "$H1TITLE" ]; then
+    AC_SUB 'H1TITLE' h1title.o
+    AC_DEFINE USE_H1TITLE 1
+else
+    AC_SUB 'H1TITLE' ''
+fi
+
+if [ "$GITHUB_CHECKBOX_STYLE" = "entity" ]; then
+    AC_DEFINE 'CHECKBOX_AS_INPUT' '0'
+else
+    AC_DEFINE 'CHECKBOX_AS_INPUT' '1'
+fi
+
+
 [ "$OS_FREEBSD" -o "$OS_DRAGONFLY" ] || AC_CHECK_HEADERS malloc.h
 
 [ "$WITH_PANDOC_HEADER" ] && AC_DEFINE 'PANDOC_HEADER' '1'
@@ -185,3 +242,16 @@ if [ "$PKGCONFIG" ]; then
 fi
 
 AC_OUTPUT $GENERATE
+
+if [ "$CXX_BINDING" ]; then
+    LOG "applying c++ glue to mkdio.h"
+    mv mkdio.h mkdio.h$$
+    (   echo '#ifdef __cplusplus'
+	echo 'extern "C" {'
+	echo '#endif'
+	cat mkdio.h$$
+	echo '#ifdef __cplusplus'
+	echo '}'
+	echo '#endif' ) > mkdio.h
+    rm mkdio.h$$
+fi
