@@ -1,7 +1,7 @@
 /*
  * theme:  use a template to create a webpage (markdown-style)
  *
- * usage:  theme [-d root] [-p pagename] [-t template] [-o html] [source]
+ * usage:  theme [-m file] [-d root] [-p pagename] [-t template] [-o html] [source]
  *
  */
 /*
@@ -36,6 +36,7 @@
 #include "gethopt.h"
 
 char *pgm = "theme";
+char *m4_file = 0;
 char *output_file = 0;
 char *pagename = 0;
 char *root = 0;
@@ -93,7 +94,7 @@ pushd(char *d)
 {
     HERE cwd;
     int size;
-    
+
     if ( chdir(d) == -1 )
 	return NOT_HERE;
 
@@ -191,7 +192,7 @@ prepare(FILE *input)
 }
 
 static int
-pull()
+pull(void)
 {
     return psp < S(inbuf) ? T(inbuf)[psp++] : EOF;
 }
@@ -215,7 +216,7 @@ shift(int shiftwidth)
 }
 
 static int*
-cursor()
+cursor(void)
 {
     return T(inbuf) + psp;
 }
@@ -253,10 +254,32 @@ istag(int *p, char *pat)
 }
 
 
+
+/* set up flags for a specific <?theme inclusion
+ */
+static void
+prepare_flags(mkd_flag_t *flags, int where)
+{
+    if ( where & INTAG )
+	mkd_set_flag_num(flags, MKD_TAGTEXT);
+    else
+	mkd_clr_flag_num(flags, MKD_TAGTEXT);
+
+    if ( where & INHEAD ) {
+	mkd_set_flag_num(flags, MKD_NOIMAGE);
+	mkd_set_flag_num(flags, MKD_NOLINKS);
+    }
+    else {
+	mkd_clr_flag_num(flags, MKD_NOIMAGE);
+	mkd_clr_flag_num(flags, MKD_NOLINKS);
+    }
+}
+
+
 /* finclude() includes some (unformatted) source
  */
 static void
-finclude(MMIOT *doc, FILE *out, int flags, int whence)
+finclude(MMIOT *doc, FILE *out, mkd_flag_t *flags, int whence)
 {
     int c;
     Cstring include;
@@ -287,7 +310,7 @@ finclude(MMIOT *doc, FILE *out, int flags, int whence)
 /* fdirname() prints out the directory part of a path
  */
 static void
-fdirname(MMIOT *doc, FILE *output, int flags, int whence)
+fdirname(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     char *p;
 
@@ -299,7 +322,7 @@ fdirname(MMIOT *doc, FILE *output, int flags, int whence)
 /* fbasename() prints out the file name part of a path
  */
 static void
-fbasename(MMIOT *doc, FILE *output, int flags, int whence)
+fbasename(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     char *p;
 
@@ -318,7 +341,7 @@ fbasename(MMIOT *doc, FILE *output, int flags, int whence)
 /* ftitle() prints out the document title
  */
 static void
-ftitle(MMIOT *doc, FILE* output, int flags, int whence)
+ftitle(MMIOT *doc, FILE* output, mkd_flag_t *flags, int whence)
 {
     char *h;
     h = mkd_doc_title(doc);
@@ -339,19 +362,22 @@ ftitle(MMIOT *doc, FILE* output, int flags, int whence)
 /* fdate() prints out the document date
  */
 static void
-fdate(MMIOT *doc, FILE *output, int flags, int whence)
+fdate(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     char *h;
 
-    if ( (h = mkd_doc_date(doc)) || ( infop && (h = ctime(&infop->st_mtime)) ) )
-	mkd_generateline(h, strlen(h), output, flags|MKD_TAGTEXT);
+    if ( (h = mkd_doc_date(doc)) || ( infop && (h = ctime(&infop->st_mtime)) ) ) {
+	mkd_set_flag_num(flags, MKD_TAGTEXT);
+	mkd_generateline(h, strlen(h), output, flags);
+	mkd_clr_flag_num(flags, MKD_TAGTEXT);
+    }
 }
 
 
 /* fauthor() prints out the document author
  */
 static void
-fauthor(MMIOT *doc, FILE *output, int flags, int whence)
+fauthor(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     char *h = mkd_doc_author(doc);
 
@@ -365,20 +391,19 @@ fauthor(MMIOT *doc, FILE *output, int flags, int whence)
 }
 
 
-/* fconfig() prints out a tabular version of
- * tabular versions of the flags.
+/* fconfig() prints out a tabular version of the flags
  */
 static void
-fconfig(MMIOT *doc, FILE *output, int flags, int whence)
+fconfig(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
-    mkd_mmiot_flags(output, doc, (whence & (INHEAD|INTAG)) ? 0 : 1);
+    mkd_flags_are(output, flags, (whence & (INHEAD|INTAG)) ? 0 : 1);
 }
 
 
 /* fversion() prints out the document version
  */
 static void
-fversion(MMIOT *doc, FILE *output, int flags, int whence)
+fversion(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     fwrite(markdown_version, strlen(markdown_version), 1, output);
 }
@@ -387,7 +412,7 @@ fversion(MMIOT *doc, FILE *output, int flags, int whence)
 /* fbody() prints out the document
  */
 static void
-fbody(MMIOT *doc, FILE *output, int flags, int whence)
+fbody(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     mkd_generatehtml(doc, output);
 }
@@ -395,7 +420,7 @@ fbody(MMIOT *doc, FILE *output, int flags, int whence)
 /* ftoc() prints out the table of contents
  */
 static void
-ftoc(MMIOT *doc, FILE *output, int flags, int whence)
+ftoc(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     mkd_generatetoc(doc, output);
 }
@@ -403,7 +428,7 @@ ftoc(MMIOT *doc, FILE *output, int flags, int whence)
 /* fstyle() prints out the document's style section
  */
 static void
-fstyle(MMIOT *doc, FILE *output, int flags, int whence)
+fstyle(MMIOT *doc, FILE *output, mkd_flag_t *flags, int whence)
 {
     mkd_generatecss(doc, output);
 }
@@ -425,7 +450,7 @@ fstyle(MMIOT *doc, FILE *output, int flags, int whence)
 static struct _keyword {
     char *kw;
     int where;
-    void (*what)(MMIOT*,FILE*,int,int);
+    void (*what)(MMIOT*,FILE*,mkd_flag_t *,int);
 } keyword[] = { 
     { "author?>",  0xffff, fauthor },
     { "body?>",    INBODY, fbody },
@@ -442,14 +467,15 @@ static struct _keyword {
 #define NR(x)	(sizeof x / sizeof x[0])
 
 
+
+
 /* spin() - run through the theme template, looking for <?theme expansions
  */
 void
-spin(FILE *template, MMIOT *doc, FILE *output)
+spin(FILE *template, MMIOT *doc, mkd_flag_t *flags, FILE *output)
 {
     int c;
     int *p;
-    int flags;
     int where = 0x0;
     int i;
 
@@ -473,17 +499,12 @@ spin(FILE *template, MMIOT *doc, FILE *output)
 		shift(-1);
 		p = cursor();
 
-		if ( where & INTAG ) 
-		    flags = MKD_TAGTEXT;
-		else if ( where & INHEAD )
-		    flags = MKD_NOIMAGE|MKD_NOLINKS;
-		else
-		    flags = 0;
-
 		for (i=0; i < NR(keyword); i++)
 		    if ( thesame(p, keyword[i].kw) ) {
-			if ( everywhere || (keyword[i].where & where) )
+			if ( everywhere || (keyword[i].where & where) ) {
+			    prepare_flags(flags,where);
 			    (*keyword[i].what)(doc,output,flags,where);
+			}
 			break;
 		    }
 
@@ -519,6 +540,7 @@ struct h_opt opts[] = {
     { 0, 0, 'd', "dir",    "set the document root" },
     { 0, 0, 'E', 0,        "do all theme expansions everywhere" },
     { 0, 0, 'f', 0,        "forcibly overwrite existing html files" },
+    { 0, 0, 'm', "file",   "preprocess the input with m4, with `file` containing preprocessing directives" },
     { 0, 0, 'o', "file",   "write output to `file`" },
     { 0, 0, 'p', "title",  "set the page title" },
     { 0, 0, 't', "template",  "use `template` as template file" },
@@ -526,24 +548,47 @@ struct h_opt opts[] = {
 } ;
 #define NROPTS (sizeof opts / sizeof opts[0])
 
-main(argc, argv)
-char **argv;
+int
+main(int argc, char **argv)
 {
     char *template = "page.theme";
     char *source = "stdin";
     FILE *tmplfile;
-    mkd_flag_t flags = THEME_CF|MKD_TOC;
+    FILE *input;
     int force = 0;
     MMIOT *doc;
     struct stat sourceinfo;
     char *q;
+    int show_version = 0; /* 0: run the program, 1: show version #, 2: show version# and initial flags */
 
     struct h_opt *opt;
     struct h_context blob;
 
+    mkd_flag_t *flags = mkd_flags();
+
+    if ( flags ) {
+#if THEME_DL_MODE > 0
+	switch (THEME_DL_MODE) {
+	case 3: mkd_set_flag_num(flags, MKD_DLEXTRA);
+	case 1: mkd_set_flag_num(flags, MKD_DLDISCOUNT);
+		break;
+	case 2: mkd_set_flag_num(flags, MKD_DLEXTRA);
+		break;
+	}
+#endif
+#ifdef THEME_FENCED_CODE
+	mkd_set_flag_num(flags, MKD_FENCEDCODE);
+#endif
+	mkd_set_flag_num(flags, MKD_TOC);
+    }
+    else {
+	perror("mkd_flags");
+	exit(1);
+    }
+
     hoptset(&blob, argc, argv);
     hopterr(&blob, 1);
-    
+
     pgm = basename(argv[0]);
 
     while ( opt = gethopt(&blob, opts, NROPTS) ) {
@@ -556,33 +601,36 @@ char **argv;
 		    break;
 	case 'E':   everywhere = 1;
 		    break;
+	case 'm':   m4_file = hoptarg(&blob);
+		    break;
 	case 'p':   pagename = hoptarg(&blob);
 		    break;
 	case 'f':   force = 1;
 		    break;
 	case 't':   template = hoptarg(&blob);
 		    break;
-	case 'C':   if ( strcmp(hoptarg(&blob), "?") == 0 ) {
-			show_flags(0,0);
+	case 'c':   if ( hoptarg(&blob)[0] == '?' ) {
+			show_flags(1, 0, hoptarg(&blob)[1] == '?' ? flags : 0);
 			exit(0);
 		    }
-		    else
-			flags = strtol(hoptarg(&blob), 0, 0);
-		    break;
-	case 'c':   if ( strcmp(hoptarg(&blob), "?") == 0 ) {
-			show_flags(1,0);
-			exit(0);
-		    }
-		    else if ( q = set_flag(&flags, hoptarg(&blob)) )
+		    else if ( q = mkd_set_flag_string(flags, hoptarg(&blob)) )
 			fprintf(stderr,"%s: unknown option <%s>", pgm, q);
 		    break;		    
 	case 'o':   output_file = hoptarg(&blob);
 		    break;
-	case 'V':   printf("theme+discount %s\n", markdown_version);
-		    exit(0);
+	case 'V':   show_version++;
+		    break;
 	}
     }
 
+    if ( show_version ) {
+	fprintf(stderr, "theme+discount %s\n", markdown_version);
+	if ( show_version > 1 )
+	    show_flags(1, 0, flags);
+	exit(0);
+    }
+
+
     tmplfile = open_template(template);
 
     argc -= hoptind(&blob);
@@ -609,7 +657,7 @@ char **argv;
 	if ( !output_file ) {
 	    char *p, *q;
 
-	    
+
 	    if ( (output_file = malloc(strlen(source) + strlen(".html") + 1)) == 0 )
 		fail("out of memory allocating output file name buffer");
 
@@ -639,7 +687,24 @@ char **argv;
     if ( !pagename )
 	pagename = source;
 
-    if ( (doc = mkd_in(stdin, 0)) == 0 )
+    if ( m4_file ) {
+	char *m4_command_line;
+	int len=0;
+
+	len = 8; /* length of 'm4 "',  '" -' & '\0' */
+
+	if ( (m4_command_line = malloc(len + strlen(m4_file))) == NULL )
+	    fail("can't allocate temporary storage?");
+
+	sprintf(m4_command_line, "m4 \"%s\" -", m4_file);
+
+	if ( (input = popen(m4_command_line, "r")) == NULL )
+	    fail("can't run m4 preprocessor (%s)", strerror(errno));
+    }
+    else
+	input = stdin;
+
+    if ( (doc = mkd_in(input, 0)) == 0 )
 	fail("can't read %s", source ? source : "stdin");
 
     if ( fstat(fileno(stdin), &sourceinfo) == 0 )
@@ -656,10 +721,12 @@ char **argv;
 	fail("couldn't compile input");
 
     if ( tmplfile )
-	spin(tmplfile,doc,stdout);
+	spin(tmplfile, doc, flags, stdout);
     else
 	mkd_generatehtml(doc, stdout);
 
     mkd_cleanup(doc);
+    mkd_free_flags(flags);
     exit(0);
 }
+
